'use strict';

module.exports = {
    SET_TOPIC: 'SET_TOPIC',

    /**
     * Action creator for setting the topic
     */
    setTopic: function setTopic(_ref) {
        var chan = _ref.chan;
        var topic = _ref.topic;

        return {
            type: module.exports.SET_TOPIC,
            chan: chan,
            topic: topic
        };
    },


    SET_TOPIC_WHO: 'SET_TOPIC_WHO',

    /**
     * Action creator for setting the topic who and time
     */
    setTopicWho: function setTopicWho(_ref2) {
        var chan = _ref2.chan;
        var hostmask = _ref2.hostmask;
        var time = _ref2.time;

        return {
            type: module.exports.SET_TOPIC_WHO,
            chan: chan,
            hostmask: hostmask,
            time: time
        };
    },


    UPDATE_NAMES: 'UPDATE_NAMES',

    /**
     * Action creator for updating channel users from NAMES reply.
     */
    updateNames: function updateNames(_ref3) {
        var chan = _ref3.chan;
        var names = _ref3.names;

        return {
            type: module.exports.UPDATE_NAMES,
            chan: chan,
            names: names
        };
    },


    USER_JOIN: 'USER_JOIN',

    /**
     * Action creator for a user joining a channel.
     */
    userJoin: function userJoin(_ref4) {
        var chan = _ref4.chan;
        var nick = _ref4.nick;
        var me = _ref4.me;

        return {
            type: module.exports.USER_JOIN,
            chan: chan,
            nick: nick,
            me: me
        };
    },


    USER_LEAVE: 'USER_LEAVE',

    /**
     * Action creator for a user leaving a channel.
     */
    userLeave: function userLeave(_ref5) {
        var chan = _ref5.chan;
        var nick = _ref5.nick;
        var me = _ref5.me;

        return {
            type: module.exports.USER_LEAVE,
            chan: chan,
            nick: nick,
            me: me
        };
    },


    USER_QUIT: 'USER_QUIT',

    /**
     * Action creator for a user quitting a channel.
     */
    userQuit: function userQuit(_ref6) {
        var nick = _ref6.nick;
        var channels = _ref6.channels;

        return {
            type: module.exports.USER_QUIT,
            nick: nick,
            channels: channels
        };
    },


    ADD_CHANNEL_MODE: 'ADD_CHANNEL_MODE',

    /**
     * Action creator for adding a mode to a channel.
     * @param {Object} state The current state
     * @param {Object} e The +mode event object from squelch-client
     */
    addChannelMode: function addChannelMode(state, _ref7) {
        var chan = _ref7.chan;
        var mode = _ref7.mode;
        var param = _ref7.param;

        return {
            type: module.exports.ADD_CHANNEL_MODE,
            prefix: state.iSupport['PREFIX'][mode] || undefined,
            chan: chan,
            mode: mode,
            param: param
        };
    },


    REMOVE_CHANNEL_MODE: 'REMOVE_CHANNEL_MODE',

    /**
     * Action creator for removing a mode to a channel.
     * @param {Object} state The current state
     * @param {Object} e The -mode event object from squelch-client
     */
    removeChannelMode: function removeChannelMode(state, _ref8) {
        var chan = _ref8.chan;
        var mode = _ref8.mode;
        var param = _ref8.param;

        return {
            type: module.exports.REMOVE_CHANNEL_MODE,
            prefix: state.iSupport['PREFIX'][mode] || undefined,
            chan: chan,
            mode: mode,
            param: param
        };
    },


    REMOVE_CHANNEL: 'REMOVE_CHANNEL',

    /**
     * Action creator for removing the data for a channel.
     */
    removeChannel: function removeChannel(_ref9) {
        var chan = _ref9.chan;

        return {
            type: module.exports.REMOVE_CHANNEL,
            chan: chan
        };
    },


    SET_ISUPPORT: 'SET_ISUPPORT',

    /**
     * Action creator for a updating iSupport values.
     * Accepts parsed reply from squelch-client raw event
     */
    setISupport: function setISupport(_ref10) {
        var params = _ref10.params;

        var iSupport = {};
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = params.slice(1)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var item = _step.value;

                // Skip the last param, which is usually something like
                // ":is supported by this server"
                if (item.indexOf(' ') !== -1) break;

                var split = item.split('=');
                if (split.length === 1) {
                    iSupport[item] = true;
                } else if (split[0] === 'PREFIX') {
                    iSupport['PREFIX'] = {};
                    var match = /\((.+)\)(.+)/.exec(split[1]);
                    for (var i = 0; i < match[1].length; i++) {
                        iSupport['PREFIX'][match[1][i]] = match[2][i];
                    }
                } else if (split[0] === 'CHANMODES') {
                    iSupport['CHANMODES'] = split[1].split(',');
                    // CHANMODES[0,1] always require param
                    // CHANMODES[2] requires param on set
                    // CHANMODES[3] never require param
                } else {
                    iSupport[split[0]] = split[1];
                }
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }

        return {
            type: module.exports.SET_ISUPPORT,
            iSupport: iSupport
        };
    },


    SET_CONNECTED: 'SET_CONNECTED',

    /**
     * Action creator for setting the connected status.
     */
    setConnected: function setConnected(connected) {
        return {
            type: module.exports.SET_CONNECTED,
            connected: connected
        };
    },


    SET_CONNECTING: 'SET_CONNECTING',

    /**
     * Action creator for setting the connecting status.
     */
    setConnecting: function setConnecting(connecting) {
        return {
            type: module.exports.SET_CONNECTING,
            connecting: connecting
        };
    },


    CHANGE_NICK: 'CHANGE_NICK',

    changeNick: function changeNick(_ref11) {
        var oldNick = _ref11.oldNick;
        var newNick = _ref11.newNick;

        return {
            type: module.exports.CHANGE_NICK,
            oldNick: oldNick,
            newNick: newNick
        };
    },


    DISCONNECT: 'DISCONNECT',

    disconnect: function disconnect() {
        return {
            type: module.exports.DISCONNECT
        };
    }
};