'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var omit = require('lodash.omit');

var _require = require('./actions');

var SET_TOPIC = _require.SET_TOPIC;
var SET_TOPIC_WHO = _require.SET_TOPIC_WHO;
var UPDATE_NAMES = _require.UPDATE_NAMES;
var USER_JOIN = _require.USER_JOIN;
var USER_LEAVE = _require.USER_LEAVE;
var USER_QUIT = _require.USER_QUIT;
var CHANGE_NICK = _require.CHANGE_NICK;
var ADD_CHANNEL_MODE = _require.ADD_CHANNEL_MODE;
var REMOVE_CHANNEL_MODE = _require.REMOVE_CHANNEL_MODE;
var DISCONNECT = _require.DISCONNECT;


module.exports = function (state, action) {
    if (!state) {
        if (!(action.type === USER_JOIN && action.me)) {
            throw new Error('Can only instantiate channel state on self USER_JOIN');
        }
        state = {
            joined: true,
            topic: '',
            topicwho: '',
            topictime: null,
            mode: [],
            users: {}
        };
    }

    switch (action.type) {
        case USER_JOIN:
            if (action.me) {
                return _extends({}, state, {
                    joined: true,
                    users: _defineProperty({}, action.nick, '')
                });
            }

            return _extends({}, state, {
                users: _extends({}, state.users, _defineProperty({}, action.nick, ''))
            });

        case USER_LEAVE:
        case USER_QUIT:
        case DISCONNECT:
            if (action.me || action.type === DISCONNECT) {
                return _extends({}, state, {
                    joined: false,
                    topic: '',
                    topicwho: '',
                    topictime: null,
                    mode: [],
                    users: {}
                });
            }

            if (!state.users[action.nick]) break;
            return _extends({}, state, {
                users: omit(state.users, action.nick)
            });

        case SET_TOPIC:
            return _extends({}, state, {
                topic: action.topic
            });

        case SET_TOPIC_WHO:
            return _extends({}, state, {
                topicwho: action.hostmask,
                topictime: action.time
            });

        case UPDATE_NAMES:
            return _extends({}, state, {
                users: action.names
            });

        case CHANGE_NICK:
            if (!state.users[action.oldNick]) break;
            return _extends({}, state, {
                users: _extends({}, omit(state.users, action.oldNick), _defineProperty({}, action.newNick, state.users[action.oldNick]))
            });

        case ADD_CHANNEL_MODE:
            if (action.prefix) {
                return _extends({}, state, {
                    users: _extends({}, state.users, _defineProperty({}, action.param, action.prefix))
                });
            }
            return _extends({}, state, {
                mode: [].concat(_toConsumableArray(state.mode), [action.mode])
            });

        case REMOVE_CHANNEL_MODE:
            if (action.prefix) {
                return _extends({}, state, {
                    users: _extends({}, state.users, _defineProperty({}, action.param, ''))
                });
            }
            return _extends({}, state, {
                mode: state.mode.filter(function (m) {
                    return m !== action.mode;
                })
            });
    }

    return state;
};