'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var omit = require('lodash.omit');
var mapValues = require('lodash.mapvalues');

var channel = require('./channelReducer');

var _require = require('./actions');

var USER_JOIN = _require.USER_JOIN;
var REMOVE_CHANNEL = _require.REMOVE_CHANNEL;
var SET_ISUPPORT = _require.SET_ISUPPORT;
var SET_CONNECTED = _require.SET_CONNECTED;
var SET_CONNECTING = _require.SET_CONNECTING;
var DISCONNECT = _require.DISCONNECT;


module.exports = function (state, action) {
    if (!state) {
        state = {
            connected: false,
            connecting: false,
            iSupport: {
                PREFIX: {
                    o: '@',
                    v: '+'
                },
                CHANTYPES: '&#',
                CHANMODES: ['beI', 'k', 'l', 'aimnpqsrt']
            },
            channels: {}
        };
    }
    switch (action.type) {
        // Create channel if we join a channel
        case USER_JOIN:
            if (action.me) {
                return _extends({}, state, {
                    channels: _extends({}, state.channels, _defineProperty({}, action.chan, channel(undefined, action)))
                });
            }
            break;

        // Delete channel data
        case REMOVE_CHANNEL:
            if (state.channels[action.chan] && state.channels[action.chan].joined) {
                throw new Error('Cannot remove a channel that is still joined.');
            }
            return _extends({}, state, {
                channels: omit(state.channels, action.chan)
            });

        case SET_ISUPPORT:
            return _extends({}, state, {
                iSupport: _extends({}, state.iSupport, action.iSupport)
            });

        case SET_CONNECTED:
            return _extends({}, state, {
                connected: action.connected
            });

        case SET_CONNECTING:
            return _extends({}, state, {
                connecting: action.connecting
            });

        case DISCONNECT:
            return _extends({}, state, {
                connecting: false,
                connected: false,
                channels: mapValues(state.channels, function (ch) {
                    return channel(ch, action);
                })
            });
    }

    // Non chan-specific action, send to all channels
    if (!action.chan) {
        return _extends({}, state, {
            channels: mapValues(state.channels, function (ch) {
                return channel(ch, action);
            })
        });
    }

    return _extends({}, state, {
        channels: _extends({}, state.channels, _defineProperty({}, action.chan, channel(state.channels[action.chan], action)))
    });
};